# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Business Applications
#    Copyright (C) 2004-2012 OpenERP S.A. (<http://openerp.com>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import logging

from datetime import datetime, timedelta
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, DATETIME_FORMATS_MAP, float_compare
from openerp.osv import fields, osv
from openerp.tools.safe_eval import safe_eval as eval
from openerp.tools.translate import _
import openerp.addons.decimal_precision as dp
from openerp import workflow
import pytz
from openerp import SUPERUSER_ID



_logger = logging.getLogger(__name__)


class terranostravinos_configuration(osv.TransientModel):
    _name = 'terranostravinos.config.settings'
    _inherit = 'res.config.settings'
    
    _columns = {
        'porcentaje_comision': fields.float('Company commission (%)', digits_compute= dp.get_precision('Discount'), implied_group='sale.group_analytic_accounting',),
        'invoice_product_id': fields.many2one('product.product', 'Commission Invoice Product', domain=[('sale_ok', '=', True)],required=True),
        'dias_aviso_entrega': fields.float('Days for delivery warning', size=3, implied_group='sale.group_analytic_accounting'),
    }

    def get_default_terranostravinos_config(self, cr, uid, ids, context=None):
        values = {}
        ir_values = self.pool.get('ir.values')        
        values['porcentaje_comision'] = ir_values.get_default(cr, uid, 'terranostravinos', 'porcentaje_comision') or 10.0                
        values['invoice_product_id'] = ir_values.get_default(cr, uid, 'terranostravinos', 'invoice_product_id') or 1                
        values['dias_aviso_entrega'] = ir_values.get_default(cr, uid, 'terranostravinos', 'dias_aviso_entrega') or 7
        return values

    def set_terranostravinos_defaults(self, cr, uid, ids, context=None):
        ir_values = self.pool.get('ir.values')
        wizard = self.browse(cr, uid, ids)[0]
        ir_values.set_default(cr, SUPERUSER_ID, 'terranostravinos', 'porcentaje_comision', wizard.porcentaje_comision)
        ir_values.set_default(cr, SUPERUSER_ID, 'terranostravinos', 'invoice_product_id', wizard.invoice_product_id.id)
        ir_values.set_default(cr, SUPERUSER_ID, 'terranostravinos', 'dias_aviso_entrega', wizard.dias_aviso_entrega)
        return {}    
    
    _defaults = {
        'porcentaje_comision': 10.0,
    }
terranostravinos_configuration()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
