# -*- coding: utf-8 -*-
'''
Created on 27 Oct 2014

@author: francesc
'''


from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, DATETIME_FORMATS_MAP, float_compare
import time
from openerp.osv import fields, osv, orm
from openerp.tools.safe_eval import safe_eval as eval
from openerp.tools.translate import _
import openerp.addons.decimal_precision as dp
from openerp import workflow
import pytz
from openerp import SUPERUSER_ID



class sale_order(osv.osv):
    _inherit = "sale.order"


    def _amount_all_wrapper(self, cr, uid, ids, field_name, arg, context=None):
        """ Wrapper because of direct method passing as parameter for function fields """
        return self._amount_all(cr, uid, ids, field_name, arg, context=context)

    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        cur_obj = self.pool.get('res.currency')
        lines_obj = self.pool.get('sale.order.line')
        por_comi = self.pool.get('ir.values').get_default(cr, uid, 'terranostravinos', 'porcentaje_comision') or 10.0          
        res = {}
        for order in self.browse(cr, uid, ids, context=context):
            res[order.id] = {
                'amount_untaxed': 0.0,
                'amount_tax': 0.0,
                'amount_total': 0.0,
                'amount_comision' : 0.0,
            }
            val = val1 = val2 = 0.0
            cur = order.pricelist_id.currency_id
            for line in order.order_line:
                val1 += line.price_subtotal
                val2 += lines_obj._amount_comision(cr, uid, [line.id], '', '', context=context)[line.id]
                val += self._amount_line_tax(cr, uid, line, context=context)
            res[order.id]['amount_tax'] = cur_obj.round(cr, uid, cur, val)
            res[order.id]['amount_untaxed'] = cur_obj.round(cr, uid, cur, val1)
            res[order.id]['amount_comision'] = cur_obj.round(cr, uid, cur, val2)
            res[order.id]['amount_total'] = res[order.id]['amount_untaxed'] + res[order.id]['amount_tax']
        return res
    
    def _get_order(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('sale.order.line').browse(cr, uid, ids, context=context):
            result[line.order_id.id] = True
        return result.keys()
    
    def _get_order_agent(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('sale.order.agent').browse(cr, uid, ids, context=context):
            result[line.sale_id.id] = True
        return result.keys()

    def _aviso_entrega(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        if context is None:
            context = {}
        dias = self.pool.get('ir.values').get_default(cr, uid, 'terranostravinos', 'dias_aviso_entrega') or 7
        hoy = fields.date.context_today(self, cr, uid, context=context)
        control = datetime.strptime(hoy, '%Y-%m-%d') + relativedelta(days=dias)
        for order in self.browse(cr, uid, ids, context=context):
            res[order.id] = False
            if order.bodega_fecha_recogida:
                if datetime.strptime(order.bodega_fecha_recogida,'%Y-%m-%d') <= control:
                    res[order.id] = True
        return res

    def _factura_bodega_num(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        if context is None:
            context = {}
        for order in self.browse(cr, uid, ids, context=context):
            name = ''
            for invoice in order.bodega_factura_ids:
                if invoice.factura_bodega_num:
                    if name:
                        name = name + ','
                    res[order.id] = name + invoice.factura_bodega_num
            res[order.id] = name
        return res

    def _factura_bodega_pagada(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        if context is None:
            context = {}
        for order in self.browse(cr, uid, ids, context=context):
            pagada = 0
            total = 0
            for invoice in order.bodega_factura_ids:
                total += 1
                if invoice.pagada:
                    pagada += 1
            if pagada and pagada == total:
                res[order.id] = True
            else:
                res[order.id] = False
        return res


    _columns = {
        'bodega_id': fields.many2one('res.partner', 'Cellar', change_default=True, readonly=True, states={'draft':[('readonly',False)]}, track_visibility='always', domain=[('bodega', '=', True)]),
        'bodega_factura_ids': fields.one2many('terranostravinos.cellar.order.invoices', 'order_id', 'Cellar Invoices'),
        'factura_bodega_num' : fields.function(_factura_bodega_num, type='char', string='Cellar invoice number(s)'),
        'bodega_lugar_recogida': fields.many2one('res.partner', 'Pickup place', domain=[('type','=','delivery'),('parent_id.bodega', '=', True)]), 
        'bodega_fecha_recogida': fields.date('Pickup date'),
        'state': fields.selection([
            ('draft', 'Draft Quotation'),
            ('sent', 'Quotation Sent'),
            ('cancel', 'Cancelled'),
            ('waiting_date', 'Waiting Schedule'),
            ('progress', 'Sales Order'),
            ('cellar_order', 'Cellar Order'),
            ('cellar_invoice', 'Cellar Invoice'),
            ('cellar_paid', 'Cellar Invoice Paid'),
            ('manual', 'Sale to Invoice'),
            ('shipping_except', 'Shipping Exception'),
            ('invoice_except', 'Invoice Exception'),
            ('done', 'Done'),
            ('cellar_done', 'Done'),
            ], 'Status', readonly=True, copy=False, help="Gives the status of the quotation or sales order.\
              \nThe exception status is automatically set when a cancel operation occurs \
              in the invoice validation (Invoice Exception) or in the picking list process (Shipping Exception).\nThe 'Waiting Schedule' status is set when the invoice is confirmed\
               but waiting for the scheduler to run on the order date.", select=True, track_visibility='onchange'),   
        'amount_comision': fields.function(_amount_all_wrapper, digits_compute=dp.get_precision('Account'), string='Commission Amount',
            store={
                'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['sale_agent_ids'], 10),
                'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line'], 10),
                'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
                'sale.order.agent': (_get_order_agent, ['agent_id', 'commission_id'], 10),
            },
            multi='sums', help="The amount of the commission.", track_visibility='always'),  
        'import_license': fields.char('Import License'), 
        'vat': fields.char('VAT'), 
        'accisa': fields.char('ACCISA'), 
        'tax_warehouse': fields.char('TAX Warehouse'), 
        'authorised_warehouse': fields.char('Authorised Warehouse'), 
        'tipo_pallets' : fields.char('Pallet type'),
        'contra_etiquetas' : fields.text('Back label'),
        'pegatinas_cajas' : fields.text('Box labels'),
        'tipo_de_cierre' : fields.char('Closure type'),
        'material_promocional' : fields.char('Promotional material'),
        'grupaje' : fields.char('Consolidation'),
        'grupaje_notes' : fields.text('Consolidation notes'),
        'informacion_adicional' : fields.text('Aditional information'),
        'bodega_terminos_entrega': fields.char('Pickup terms'),
        'bodega_fecha_preparacion': fields.date('Real delivery ready date'),
        'transportista_id': fields.many2one('res.partner', 'Delivery agent', domain=[('transportista', '=', True)]),
        'transportista_seguimiento' : fields.char('Transport follow up'),
        'bodega_documentacion': fields.text('Documentation'), 
        'bodega_emcs': fields.text('EMCS'), 
        'bodega_estado_pedido': fields.many2one('terranostravinos.miestado', 'Order Control State',track_visibility='onchange'),
        'bodega_aviso_entrega': fields.function(_aviso_entrega, string='Delivery warning',type="boolean",track_visibility='onchange'),
        'bodega_reenviado': fields.boolean('Already sent',track_visibility='onchange'), 
        'bodega_notasreenvio': fields.text('Already sent notes'), 
        'sale_agent_ids': fields.one2many('sale.order.agent', 'sale_id', 'Agents',readonly=True),
        'user_original_id': fields.many2one('res.users', 'Original Salesperson', states={'draft': [('readonly', False)], 'sent': [('readonly', False)]}, select=True, readonly=True, track_visibility='onchange'),
        'product_template_id': fields.related('order_line', 'product_template_id', type='many2one', relation='product.template', string='Product Template'),
#        'facturas_pagadas': fields.related('bodega_factura_ids', 'pagada', type='many2one', relation='terranostravinos.cellar.order.invoices', string='Cellar invoices paid'),
        'facturas_pagadas': fields.function(_factura_bodega_pagada, type='boolean', string='Cellar invoice(s) paid',track_visibility='onchange'),
      }
    _defaults = {    
          'user_original_id': lambda obj, cr, uid, context: uid,
          }

    def action_quotation_send(self, cr, uid, ids, context=None):
        '''
        This function opens a window to compose an email, with the edi sale template message loaded by default
        '''
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        ir_model_data = self.pool.get('ir.model.data')
        try:
            order =self.browse(cr,uid,ids[0],context=context)
            if order.bodega_id:
                template_id = ir_model_data.get_object_reference(cr, uid, 'sale', 'email_template_edi_cellar_sale')[1]
            else:
                template_id = ir_model_data.get_object_reference(cr, uid, 'sale', 'email_template_edi_sale')[1]
        except ValueError:
            template_id = False
        try:
            compose_form_id = ir_model_data.get_object_reference(cr, uid, 'mail', 'email_compose_message_wizard_form')[1]
        except ValueError:
            compose_form_id = False 
        ctx = dict()
        ctx.update({
            'default_model': 'sale.order',
            'default_res_id': ids[0],
            'default_use_template': bool(template_id),
            'default_template_id': template_id,
            'default_composition_mode': 'comment',
            'mark_so_as_sent': True
        })
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'mail.compose.message',
            'views': [(compose_form_id, 'form')],
            'view_id': compose_form_id,
            'target': 'new',
            'context': ctx,
        }


    def action_button_confirm(self, cr, uid, ids, context=None):
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        for o in self.browse(cr, uid, ids):
            if not o.order_line:
                raise osv.except_osv(_('Error!'),_('You cannot confirm a sales order which has no line.'))
            if o.bodega_id:
                self.write(cr, uid, [o.id], {'state': 'cellar_order', 'date_confirm': fields.date.context_today(self, cr, uid, context=context)})
                self.pool.get('sale.order.line').button_confirm(cr, uid, [x.id for x in o.order_line])
        self.signal_workflow(cr, uid, ids, 'order_confirm')
        return True

  
    def action_wait(self, cr, uid, ids, context=None):
        context = context or {}
        for o in self.browse(cr, uid, ids):
            if not o.order_line:
                raise osv.except_osv(_('Error!'),_('You cannot confirm a sales order which has no line.'))
            if o.bodega_id:
                self.write(cr, uid, [o.id], {'state': 'cellar_order', 'date_confirm': fields.date.context_today(self, cr, uid, context=context)})
                self.pool.get('sale.order.line').button_confirm(cr, uid, [x.id for x in o.order_line])
            else:
                noprod = self.test_no_product(cr, uid, o, context)
                if (o.order_policy == 'manual') or noprod:
                    self.write(cr, uid, [o.id], {'state': 'manual', 'date_confirm': fields.date.context_today(self, cr, uid, context=context)})
                else:
                    self.write(cr, uid, [o.id], {'state': 'progress', 'date_confirm': fields.date.context_today(self, cr, uid, context=context)})
                self.pool.get('sale.order.line').button_confirm(cr, uid, [x.id for x in o.order_line])
        return True

    def action_cellar_cancel_order(self, cr, uid, ids, context=None):
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        context = context or {}
        for o in self.browse(cr, uid, ids):
            self.write(cr, uid, [o.id], {'state': 'draft'})
            self.pool.get('sale.order.line').write(cr, uid, [x.id for x in o.order_line], {'state': 'draft'}) 
        # self.signal_workflow(cr, uid, ids, 'action_cellar_paid')
        return True
 
    def action_cellar_invoice(self, cr, uid, ids, context=None):
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        context = context or {}
        for o in self.browse(cr, uid, ids):
#             if not o.factura_bodega_num or not o.factura_bodega_date or not o.factura_bodega_venc:
#                 raise osv.except_osv(_('Error!'),_('Please, inform cellar invoice number and dates.'))
            self.write(cr, uid, [o.id], {'state': 'cellar_invoice'})
        self.signal_workflow(cr, uid, ids, 'action_cellar_invoice')
        return True

    def action_cellar_cancel_invoice(self, cr, uid, ids, context=None):
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        context = context or {}
        for o in self.browse(cr, uid, ids):
            self.write(cr, uid, [o.id], {'state': 'cellar_order'})
        # self.signal_workflow(cr, uid, ids, 'action_cellar_paid')
        return True

    def action_cellar_paid(self, cr, uid, ids, context=None):
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        context = context or {}
        for o in self.browse(cr, uid, ids):
            if not o.bodega_factura_ids:
                raise osv.except_osv(_('Error!'),_('Please, inform cellar invoice(s).'))
        self.write(cr, uid, [o.id], {'state': 'cellar_paid'})
        self.signal_workflow(cr, uid, ids, 'action_cellar_paid')
        return True

    def action_cellar_cancel_paid(self, cr, uid, ids, context=None):
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        context = context or {}
        for o in self.browse(cr, uid, ids):
            self.write(cr, uid, [o.id], {'state': 'cellar_invoice'})
        # self.signal_workflow(cr, uid, ids, 'action_cellar_paid')
        return True
    
    def action_cancel_done(self, cr, uid, ids, context=None):
        assert len(ids) == 1, 'This option should only be used for a single id at a time.'
        context = context or {}
        for o in self.browse(cr, uid, ids):
            self.write(cr, uid, [o.id], {'state': 'cellar_paid'})
        # self.signal_workflow(cr, uid, ids, 'action_cellar_paid')
        return True


    def manual_invoice(self, cr, uid, ids, context=None):
        """ create invoices for the given sales orders (ids), and open the form
            view of one of the newly created invoices
        """
        mod_obj = self.pool.get('ir.model.data')
        for o in self.browse(cr, uid, ids, context):
            if o.bodega_id:
                return False
        # create invoices through the sales orders' workflow
        inv_ids0 = set(inv.id for sale in self.browse(cr, uid, ids, context) for inv in sale.invoice_ids)
        self.signal_workflow(cr, uid, ids, 'manual_invoice')
        inv_ids1 = set(inv.id for sale in self.browse(cr, uid, ids, context) for inv in sale.invoice_ids)
        # determine newly created invoices
        new_inv_ids = list(inv_ids1 - inv_ids0)

        res = mod_obj.get_object_reference(cr, uid, 'account', 'invoice_form')
        res_id = res and res[1] or False,

        return {
            'name': _('Customer Invoices'),
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [res_id],
            'res_model': 'account.invoice',
            'context': "{'type':'out_invoice'}",
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'current',
            'res_id': new_inv_ids and new_inv_ids[0] or False,
        }


    def _prepare_invoice(self, cr, uid, order, lines, context=None):
        """Prepare the dict of values to create the new invoice for a
           sales order. This method may be overridden to implement custom
           invoice generation (making sure to call super() to establish
           a clean extension chain).

           :param browse_record order: sale.order record to invoice
           :param list(int) line: list of invoice line IDs that must be
                                  attached to the invoice
           :return: dict of value to create() the invoice
        """
        if context is None:
            context = {}
        journal_ids = self.pool.get('account.journal').search(cr, uid,
            [('type', '=', 'sale'), ('company_id', '=', order.company_id.id)],
            limit=1)
        if not journal_ids:
            raise osv.except_osv(_('Error!'),
                _('Please define sales journal for this company: "%s" (id:%d).') % (order.company_id.name, order.company_id.id))

        if order.bodega_id:
            pricelist = order.bodega_id.property_product_pricelist or False
            payment_term = order.bodega_id.property_payment_term and part.property_payment_term.id or False

            invoice_vals = {
                'name': order.client_order_ref or '',
                'origin': order.factura_bodega_num,
                'type': 'out_invoice',
                'reference': order.factura_bodega_num+' - '+order.name,
                'account_id': order.bodega_id.property_account_receivable.id,
                'partner_id': order.bodega_id.id,
                'journal_id': journal_ids[0],
                'invoice_line': [(6, 0, lines)],
                'currency_id': pricelist.currency_id.id,
                'comment': '',
                'payment_term': order.payment_term and order.payment_term.id or False,
                'fiscal_position': order.bodega_id.property_account_position.id,
                'date_invoice': context.get('date_invoice', False),
                'company_id': order.company_id.id,
                'user_id': order.user_id and order.user_id.id or False,
                'section_id' : order.section_id.id
            }
        else:
            invoice_vals = {
                'name': order.client_order_ref or '',
                'origin': order.name,
                'type': 'out_invoice',
                'reference': order.client_order_ref or order.name,
                'account_id': order.partner_id.property_account_receivable.id,
                'partner_id': order.partner_invoice_id.id,
                'journal_id': journal_ids[0],
                'invoice_line': [(6, 0, lines)],
                'currency_id': order.pricelist_id.currency_id.id,
                'comment': order.note,
                'payment_term': order.payment_term and order.payment_term.id or False,
                'fiscal_position': order.fiscal_position.id or order.partner_id.property_account_position.id,
                'date_invoice': context.get('date_invoice', False),
                'company_id': order.company_id.id,
                'user_id': order.user_id and order.user_id.id or False,
                'section_id' : order.section_id.id
            }

        # Care for deprecated _inv_get() hook - FIXME: to be removed after 6.1
        invoice_vals.update(self._inv_get(cr, uid, order, context=context))
        return invoice_vals
    
    def _make_invoice(self, cr, uid, order, lines, context=None):
        inv_obj = self.pool.get('account.invoice')
        obj_invoice_line = self.pool.get('account.invoice.line')
        if context is None:
            context = {}
        if not order.bodega_id:
            invoiced_sale_line_ids = self.pool.get('sale.order.line').search(cr, uid, [('order_id', '=', order.id), ('invoiced', '=', True)], context=context)
            from_line_invoice_ids = []
            for invoiced_sale_line_id in self.pool.get('sale.order.line').browse(cr, uid, invoiced_sale_line_ids, context=context):
                for invoice_line_id in invoiced_sale_line_id.invoice_lines:
                    if invoice_line_id.invoice_id.id not in from_line_invoice_ids:
                        from_line_invoice_ids.append(invoice_line_id.invoice_id.id)
            for preinv in order.invoice_ids:
                if preinv.state not in ('cancel',) and preinv.id not in from_line_invoice_ids:
                    for preline in preinv.invoice_line:
                        inv_line_id = obj_invoice_line.copy(cr, uid, preline.id, {'invoice_id': False, 'price_unit': -preline.price_unit})
                        lines.append(inv_line_id)
        inv = self._prepare_invoice(cr, uid, order, lines, context=context)
        inv_id = inv_obj.create(cr, uid, inv, context=context)
        data = inv_obj.onchange_payment_term_date_invoice(cr, uid, [inv_id], inv['payment_term'], time.strftime(DEFAULT_SERVER_DATE_FORMAT))
        if data.get('value', False):
            inv_obj.write(cr, uid, [inv_id], data['value'], context=context)
        inv_obj.button_compute(cr, uid, [inv_id])
        return inv_id
    
    def _create_invoice_line(self, cr, uid, order, articulo, context=None):
        
        if articulo:
            account_id = articulo.property_account_income.id
            if not account_id:
                account_id = articulo.categ_id.property_account_income_categ.id
#            if not account_id:
#                raise osv.except_osv(_('Error!'),
#                        _('Please define income account for this product: "%s" (id:%d).') % \
#                            (line.product_id.name, line.product_id.id,))
        else:
            prop = self.pool.get('ir.property').get(cr, uid,
                    'property_account_income_categ', 'product.category',
                    context=context)
            account_id = prop and prop.id or False

        pu = order.amount_comision
        
        fiscal_obj = self.pool.get('account.fiscal.position')

        account_id = fiscal_obj.map_account(cr, uid, order.bodega_id.property_account_position, account_id)
#        if not account_id:
#            raise osv.except_osv(_('Error!'),
#                        _('There is no Fiscal Position defined or Income category account defined for default properties of Product categories.'))
        vals = {
            'name': order.factura_bodega_num,
#            'sequence': line.sequence,
            'origin': order.name,
            'account_id': account_id,
            'price_unit': pu,
            'quantity': 1.0,
            'discount': 0.0,
            'uos_id': False,
            'product_id': articulo.id or False,
            'invoice_line_tax_id':[[6, 0, fiscal_obj.map_tax(cr, uid, order.bodega_id.property_account_position, articulo.taxes_id)]],
            'company_id': order.company_id.id,
            'partner_id': order.partner_id.id
        }

        # self.pool.get('sale.order.line').write(cr,uid,[x.id for x in order.order_line],{'invoiced':True},context=context)
        inv_id = self.pool.get('account.invoice.line').create(cr, uid, vals, context=context)
        self.pool.get('sale.order.line').write(cr, uid, [x.id for x in order.order_line], {'invoice_lines': [(4, inv_id)]}, context=context)
        return [inv_id]

    def action_invoice_create(self, cr, uid, ids, grouped=False, states=None, date_invoice = False, context=None):
        if states is None:
            states = ['confirmed', 'done', 'exception' , 'cellar_paid']
        res = False
        invoices = {}
        invoice_ids = []
        invoice = self.pool.get('account.invoice')
        obj_sale_order_line = self.pool.get('sale.order.line')
        partner_currency = {}
        
        articulo = self.pool.get('product.product').browse(cr,uid,self.pool.get('ir.values').get_default(cr, uid, 'terranostravinos', 'invoice_product_id') or 1,context=context)

        # If date was specified, use it as date invoiced, usefull when invoices are generated this month and put the
        # last day of the last month as invoice date
        if date_invoice:
            context = dict(context or {}, date_invoice=date_invoice)
        for o in self.browse(cr, uid, ids, context=context):
            if o.bodega_id:
                if o.state != 'cellar_paid':
                    continue
                if o.invoice_ids:
                    invoiced = False
                    for invoice in o.invoice_ids:
                        if invoice.state != 'cancel':
                            invoiced = True
                            break
                    if invoiced:
                        continue
                created_lines = self._create_invoice_line(cr, uid, o,articulo)
                if created_lines:
                    invoices.setdefault(o.bodega_id.id, []).append((o, created_lines))
            else:
                currency_id = o.pricelist_id.currency_id.id
                if (o.partner_id.id in partner_currency) and (partner_currency[o.partner_id.id] <> currency_id):
                    raise osv.except_osv(
                        _('Error!'),
                        _('You cannot group sales having different currencies for the same partner.'))

                partner_currency[o.partner_id.id] = currency_id
                lines = []
                for line in o.order_line:
                    if line.invoiced:
                        continue
                    elif (line.state in states):
                        lines.append(line.id)
                created_lines = obj_sale_order_line.invoice_line_create(cr, uid, lines)
                if created_lines:
                    invoices.setdefault(o.partner_invoice_id.id or o.partner_id.id, []).append((o, created_lines))
        if not invoices:
            for o in self.browse(cr, uid, ids, context=context):
                for i in o.invoice_ids:
                    if i.state == 'draft':
                        return i.id
        for val in invoices.values():
            if grouped:
                res = self._make_invoice(cr, uid, val[0][0], reduce(lambda x, y: x + y, [l for o, l in val], []), context=context)
                invoice_ref = ''
                origin_ref = ''
                for o, l in val:
                    if o.bodega_id:
                        invoice_ref += (o.factura_bodega_num +' - '+ o.name) + '|'
                        origin_ref += (o.factura_bodega_num or o.origin or o.name) + '|'
                        self.write(cr, uid, [o.id], {'state': 'cellar_done'})
                        cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%s,%s)', (o.id, res))
                        self.invalidate_cache(cr, uid, ['invoice_ids'], [o.id], context=context)
                    else:
                        invoice_ref += (o.client_order_ref or o.name) + '|'
                        origin_ref += (o.origin or o.name) + '|'
                        self.write(cr, uid, [o.id], {'state': 'progress'})
                        cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%s,%s)', (o.id, res))
                        self.invalidate_cache(cr, uid, ['invoice_ids'], [o.id], context=context)
                #remove last '|' in invoice_ref
                if len(invoice_ref) >= 1:
                    invoice_ref = invoice_ref[:-1]
                if len(origin_ref) >= 1:
                    origin_ref = origin_ref[:-1]
                invoice.write(cr, uid, [res], {'origin': origin_ref, 'name': invoice_ref})
            else:
                for order, il in val:
                    if o.bodega_id:
                        res = self._make_invoice(cr, uid, order, il, context=context)
                        invoice_ids.append(res)
                        self.write(cr, uid, [order.id], {'state': 'cellar_done'})
                        cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%s,%s)', (order.id, res))
                        self.invalidate_cache(cr, uid, ['invoice_ids'], [order.id], context=context)
                    else:
                        res = self._make_invoice(cr, uid, order, il, context=context)
                        invoice_ids.append(res)
                        self.write(cr, uid, [order.id], {'state': 'progress'})
                        cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%s,%s)', (order.id, res))
                        self.invalidate_cache(cr, uid, ['invoice_ids'], [order.id], context=context)
        return res

    def action_invoice_cancel(self, cr, uid, ids, context=None):
        for o in self.browse(cr, uid, ids):
            if o.bodega_id:
                self.write(cr, uid, [o.id], {'state': 'cellar_paid'})
                return True

        self.write(cr, uid, ids, {'state': 'invoice_except'}, context=context)
        return True

    def action_invoice_end(self, cr, uid, ids, context=None):
        for this in self.browse(cr, uid, ids, context=context):
            for line in this.order_line:
                if line.state == 'exception':
                    line.write({'state': 'confirmed'})
            if this.bodega_id:
                return True
            if this.state == 'invoice_except':
                this.write({'state': 'progress'})
        return True

    def print_quotation(self, cr, uid, ids, context=None):
        '''
        This function prints the sales order and mark it as sent, so that we can see more easily the next step of the workflow
        '''
        assert len(ids) == 1, 'This option should only be used for a single id at a time'
        ids_bodega = []
        ids_sale = []
        for o in self.browse(cr,uid,ids,context=context):
            if o.bodega_id:
                ids_bodega.append(o.id)
            else:
                ids_sale.append(o.id)
        if ids_bodega:
            self.signal_workflow(cr, uid, ids_bodega, 'quotation_sent')
            return self.pool['report'].get_action(cr, uid, ids_bodega, 'terranostravinos.report_cellarorder', context=context)
        self.signal_workflow(cr, uid, ids_sale, 'quotation_sent')
        return self.pool['report'].get_action(cr, uid, ids_sale, 'sale.report_saleorder', context=context)


    def onchange_partner_comision_id(self, cr, uid, ids, part, order_line = False, context=None):
        result = super(sale_order,self).onchange_partner_id(cr, uid, ids,part, context=context)
        if 'value' in result:
            if 'pricelist_id' in result['value']:
                result['value']['currency_id'] = self.pool.get('product.pricelist').browse(cr, uid, result['value']['pricelist_id'], context=context).currency_id.id
            part = self.pool.get('res.partner').browse(cr, uid, part, context=context)
            result['value']['import_license'] = part.import_license
            result['value']['vat'] = part.vat
            result['value']['accisa'] = part.accisa
            result['value']['tax_warehouse'] = part.tax_warehouse 
            result['value']['authorised_warehouse'] = part.authorised_warehouse
            result['value']['tipo_pallets'] = part.tipo_pallets
            result['value']['contra_etiquetas'] = part.contra_etiquetas
             
            if order_line and isinstance(order_line, list):
                sale_agent_ids = [(6,0,[])]
                
                if part.commission_ids:
                    for partner_agent in part.commission_ids:
                        vals = {
                            'agent_id': partner_agent.agent_id.id,
                            'commission_id': partner_agent.commission_id.id,
                        }
                        sale_agent_ids.append((0,0,vals))
                order_lines = []
                for line in order_line:
                    newline = line
                    if newline[2]:
                        if isinstance(newline[2], dict):
                            newline[2]['sale_agent_ids'] = sale_agent_ids
                            newline[2]['order_partner_id'] = part.id
                        elif isinstance(newline[2], list) and isinstance(newline[2][0], (int,long)):
                            for line_id in line[2]:
                                newline = (1,line_id,{'sale_agent_ids' : sale_agent_ids,'order_partner_id' : part.id}) 
                                order_lines.append(newline)
                            newline = False
                    if newline:
                        order_lines.append(newline)
                result['value']['order_line'] = order_lines               
        return result


    def onchange_bodega_id(self, cr, uid, ids, bodega_id, order_line, context=None):
        result = {'value':{}}
        if order_line:
            order_lines = []
            for line in order_line:
                newline = line
                if newline[2]:
                    if isinstance(newline[2], dict):
                        newline[2]['order_bodega_id'] = bodega_id
                    elif isinstance(newline[2], list) and isinstance(newline[2][0], (int,long)):
                        for line_id in line[2]:
                            newline = (1,line_id,{'order_bodega_id' : bodega_id}) 
                            order_lines.append(newline)
                        newline = False
                if newline:
                    order_lines.append(newline)
            result['value']['order_line'] = order_lines  
        return result
    
    def write(self, cr, uid, ids, vals, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]
        lines = self.pool.get('sale.order.line')
        res = super(sale_order, self).write(cr, uid, ids, vals, context=context)
        for order in self.browse(cr,uid,ids,context=context):
            lines.write(cr,uid,[line.id for line in order.order_line],{'order_bodega_id': order.bodega_id and order.bodega_id.id or False},context=context)
        return res


class sale_order_line(osv.osv):
    _inherit = 'sale.order.line'

    def product_id_change(self, cr, uid, ids, pricelist, product, qty=0,
            uom=False, qty_uos=0, uos=False, name='', partner_id=False,
            lang=False, update_tax=True, date_order=False, packaging=False, fiscal_position=False, flag=False, context=None):
        result = super(sale_order_line,self).product_id_change(cr, uid, ids, pricelist, product, qty,uom, qty_uos, uos, name, partner_id,lang, update_tax, date_order, packaging, fiscal_position, flag, context)
        por_comi = self.pool.get('ir.values').get_default(cr, uid, 'terranostravinos', 'porcentaje_comision') or 10.0          
        tabla_comision = self.pool.get('terranostravinos.commission.client.product')
        idscom = tabla_comision.search(cr, uid, [('partner_id', '=', partner_id),('product_id', '=', product)], context=context)
        if idscom:
            por_comi = tabla_comision.browse(cr,uid,idscom[0],context=context).commission
        if not 'value' in result:
            result['value'] = {}
        if product:
            partner = self.pool.get('res.partner').browse(cr, uid, partner_id)
            lang = partner.lang
            context_partner = {'lang': lang, 'partner_id': partner_id}
            product_obj = self.pool.get('product.product').browse(cr, uid, product, context=context_partner)
            result['value']['anyada'] = product_obj.anyada
            if product_obj.anyada:
                result['value']['name'] = product_obj.name_template
        result['value'].update({'company_comision' : por_comi})
        return result
    
    def _amount_comision(self, cr, uid, ids, field_name, arg, context=None):
        tax_obj = self.pool.get('account.tax')
        cur_obj = self.pool.get('res.currency')

        res = {}
        if context is None:
            context = {}
        for line in self.browse(cr, uid, ids, context=context):
            comision = 0.0
            if line.order_id.bodega_id:
                por_comi = line.company_comision
                price = line.price_unit * (1 - (line.discount or 0.0) / 100.0)
                taxes = tax_obj.compute_all(cr, uid, [], price, line.product_uom_qty, line.product_id, line.order_id.partner_id)
                cur = line.order_id.pricelist_id.currency_id
                total = taxes['total']
                for line_agent in line.sale_agent_ids:
                    if line_agent.agent_id.type != 'employed':
                        quantity = 0.0
                        if line_agent.commission_id.type == 'fijo' and line_agent.commission_id.fix_qty:
                            quantity = total * (line_agent.commission_id.fix_qty / 100.0)
                        total -= quantity
                comision = cur_obj.round(cr, uid, cur, total * (por_comi or 0.0) / 100.0)
            res[line.id] = comision
        return res
    
    
    def _get_order_agent(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('sale.order.line.agent').browse(cr, uid, ids, context=context):
            result[line.line_id.id] = True
        return result.keys()

    _columns = {
            'discount': fields.float('%Dto', digits_compute= dp.get_precision('Discount'), readonly=True, states={'draft': [('readonly', False)]}),
            'anyada': fields.char('Vtg',size=4),
            'company_comision': fields.float('%Com', digits_compute= dp.get_precision('Discount')),
            'comision_subtotal': fields.function(_amount_comision, string='Commission', digits_compute= dp.get_precision('Account'),
             store={
                'sale.order.line': (lambda self, cr, uid, ids, c={}: ids, ['company_comision','price_unit', 'tax_id', 'discount', 'product_uom_qty','sale_agent_ids'], 10),
                'sale.order.line.agent': (_get_order_agent, ['agent_id', 'commission_id'], 10),
            }
           ),
            'order_bodega_id': fields.many2one('res.partner','Cellar'),
            'sale_agent_ids': fields.one2many('sale.order.line.agent', 'line_id', 'Agents'),
            'product_template_id': fields.related('product_id', 'product_tmpl_id', type='many2one', relation='product.template', store=True, string='Product Template'),
    }
    
    def onchange_agents_ids(self, cr, uid, ids, agent_ids, context=None):
        result = {'value':{'comision_subtotal' : 0.0}}
        return result
    
    def onchange_company_comision(self, cr, uid, ids, company_comision, agent_ids, context=None):
        result = {'value':{'comision_subtotal' : 0.0}}
        return result

    def onchange_initial_defaults(self, cr, uid, ids, bodega_id, partner_id, context=None):
        result = {'value':{'order_bodega_id' : bodega_id, 'order_partner_id' : partner_id}}
        
        
        sale_agent_ids = [(6,0,[])]
        
        partner = self.pool.get('res.partner').browse(cr, uid, partner_id, context=context)
        if partner.commission_ids:
            for partner_agent in partner.commission_ids:
                vals = {
                    'agent_id': partner_agent.agent_id.id,
                    'commission_id': partner_agent.commission_id.id,
                }
                sale_agent_ids.append((0,0,vals))
        result['value']['sale_agent_ids'] = sale_agent_ids

        return result    
    
class sale_agent(orm.Model):
    _inherit = "sale.agent"
    _columns = {
        'type': fields.selection((('asesor', 'Adviser'), ('comercial', 'Commercial'), ('employed', 'Employee')), 'Type', required=True),                
    }    
    
class sale_order_agent(orm.Model):
    _inherit = "sale.order.agent"
    
    def _amount_agent_comision(self, cr, uid, ids, field_name, arg, context=None):
        tax_obj = self.pool.get('account.tax')
        cur_obj = self.pool.get('res.currency')

        res = {}
        if context is None:
            context = {}
        for agent_line in self.browse(cr, uid, ids, context=context):
            tot_quantity = 0.0
#             for line in agent_line.sale_id.order_line:
#                 price = line.price_unit * (1 - (line.discount or 0.0) / 100.0)
#                 taxes = tax_obj.compute_all(cr, uid, [], price, line.product_uom_qty, line.product_id, line.order_id.partner_id)
#                 cur = line.order_id.pricelist_id.currency_id
#                 total = taxes['total']
#                 for line_agent in line.order_id.sale_agent_ids:
#                     quantity = 0.0
#                     if line_agent.commission_id.type == 'fijo' and line_agent.commission_id.fix_qty:
#                         quantity = total * (line_agent.commission_id.fix_qty / 100.0)
#                     total -= quantity
#                     if agent_line.id == line_agent.id:
#                         tot_quantity += quantity
            res[agent_line.id] = tot_quantity
        return res
    
    _columns = {
        'sale_id': fields.many2one('sale.order', 'Sale order', required=False,  help=''),
        'agent_id': fields.many2one('sale.agent', 'Agent', required=True,  help=''),
        'commission_id': fields.many2one('commission', 'Applied commission', required=True,
                                         help=''),
        'comision_total': fields.function(_amount_agent_comision, string='Commission Total', digits_compute= dp.get_precision('Account')),
    }

    def onchange_agent_id(self, cr, uid, ids, agent_id, context=None):
        result = super(sale_order_agent, self).onchange_agent_id(cr, uid, ids,agent_id, context=context)
        if not 'value' in result:
            result['value'] = {}
        result['value']['comision_total'] = 0.0
        return result

    def onchange_commission_id(self, cr, uid, ids, agent_id=False, commission_id=False, context=None):
        result = super(sale_order_agent, self).onchange_commission_id(cr, uid, ids,agent_id=agent_id,commission_id=commission_id,context=context)
        if not 'value' in result:
            result['value'] = {}
        result['value']['comision_total'] = 0.0
        return result

class sale_order_line_agent(orm.Model):
    _name = "sale.order.line.agent"

    def name_get(self, cr, uid, ids, context=None):
        """devuelve como nombre del agente del partner el nombre del agente"""
        if context is None:
            context = {}
        return [(obj.id, obj.agent_id.name) for obj in self.browse(cr, uid, ids, context=context)]

    def _amount_agent_comision(self, cr, uid, ids, field_name, arg, context=None):
        tax_obj = self.pool.get('account.tax')
        cur_obj = self.pool.get('res.currency')

        res = {}
        if context is None:
            context = {}
        for agent_line in self.browse(cr, uid, ids, context=context):
            tot_quantity = 0.0
            line = agent_line.line_id

            price = line.price_unit * (1 - (line.discount or 0.0) / 100.0)
            taxes = tax_obj.compute_all(cr, uid, [], price, line.product_uom_qty, line.product_id, line.order_id.partner_id)
            cur = line.order_id.pricelist_id.currency_id
            total = taxes['total']
            for line_agent in line.sale_agent_ids:
                quantity = 0.0
                if line_agent.commission_id.type == 'fijo' and line_agent.commission_id.fix_qty:
                    quantity = total * (line_agent.commission_id.fix_qty / 100.0)
                    total -= quantity
                    if agent_line.id == line_agent.id:
                        tot_quantity += quantity
            res[agent_line.id] = tot_quantity
        return res

    _columns = {
        'line_id': fields.many2one('sale.order.line', 'Sale order line', required=False, help=''),
        'agent_id': fields.many2one('sale.agent', 'Agent', required=True, help=''),
        'commission_id': fields.many2one('commission', 'Applied commission', required=True,
                                         help=''),
        'comision_total': fields.function(_amount_agent_comision, string='Commission Total', digits_compute= dp.get_precision('Account')),
    }

    def onchange_agent_id(self, cr, uid, ids, agent_id, partner_id, product, context=None):
        """al cambiar el agente cargamos sus comisión"""
        if context is None:
            context = {}
        result = {}
        v = {}
        if agent_id:
            agent = self.pool.get('sale.agent').browse(cr, uid, agent_id, context=context)
            idcomi = agent.commission.id
            tabla_comision = self.pool.get('terranostravinos.commission.client.product.agent')
            idscom = tabla_comision.search(cr, uid, [('partner_id', '=', partner_id),('product_id', '=', product),('agent_id', '=', agent_id)], context=context)
            if idscom:
                idcomi = tabla_comision.browse(cr,uid,idscom[0],context=context).commission.id
            v['commission_id'] = idcomi
            v['comision_total'] = 0.0
        result['value'] = v
        return result

    def onchange_commission_id(self, cr, uid, ids, agent_id, commission_id, context=None):
        """al cambiar la comisión comprobamos la selección"""
        if context is None:
            context = {}
        result = {}

        if commission_id:
            result['value'] = {'comision_total' : 0.0}
            partner_commission = self.pool.get('commission').browse(cr, uid, commission_id, context=context)
            if partner_commission.sections and agent_id:
                agent = self.pool.get('sale.agent').browse(cr, uid, agent_id, context=context)
                if agent.commission.id != partner_commission.id:
                    result['warning'] = {
                        'title': _('Fee installments!'),
                        'message': _('A commission has been assigned by sections that does not '
                                     'match that defined for the agent by default, so that these '
                                     'sections shall apply only on this bill.')
                    }
        return result

class sale_make_invoice(osv.osv_memory):
    _inherit = "sale.make.invoice"
    
    def make_invoices(self, cr, uid, ids, context=None):
        order_obj = self.pool.get('sale.order')
        mod_obj = self.pool.get('ir.model.data')
        act_obj = self.pool.get('ir.actions.act_window')
        newinv = []
        if context is None:
            context = {}
        data = self.read(cr, uid, ids)[0]
        for sale_order in order_obj.browse(cr, uid, context.get(('active_ids'), []), context=context):
            if not sale_order.state in ('manual','cellar_paid'):
                raise osv.except_osv(_('Warning!'), _("You shouldn't manually invoice the following sale order %s") % (sale_order.name))

        order_obj.action_invoice_create(cr, uid, context.get(('active_ids'), []), data['grouped'], date_invoice=data['invoice_date'])
        orders = order_obj.browse(cr, uid, context.get(('active_ids'), []), context=context)
        for o in orders:
            for i in o.invoice_ids:
                newinv.append(i.id)
        # Dummy call to workflow, will not create another invoice but bind the new invoice to the subflow
        order_obj.signal_workflow(cr, uid, [o.id for o in orders if o.order_policy == 'manual'], 'manual_invoice')
        result = mod_obj.get_object_reference(cr, uid, 'account', 'action_invoice_tree1')
        id = result and result[1] or False
        result = act_obj.read(cr, uid, [id], context=context)[0]
        result['domain'] = "[('id','in', [" + ','.join(map(str, newinv)) + "])]"

        return result
    
class product_product(osv.Model):
    _inherit = 'product.product'

    def _sales_count(self, cr, uid, ids, field_name, arg, context=None):
        SaleOrderLine = self.pool['sale.order.line']
        return {
            product_id: SaleOrderLine.search_count(cr,uid, [('product_id', '=', product_id),('order_bodega_id','=',False)], context=context)
            for product_id in ids
        }

    def _cellar_count(self, cr, uid, ids, field_name, arg, context=None):
        SaleOrderLine = self.pool['sale.order.line']
        return {
            product_id: SaleOrderLine.search_count(cr,uid, [('product_id', '=', product_id),('order_bodega_id','!=',False)], context=context)
            for product_id in ids
        }

    _columns = {
        'sales_count': fields.function(_sales_count, string='# Sales', type='integer'),
        'cellar_count': fields.function(_cellar_count, string='# Cellar', type='integer'),
        'anyada': fields.char('Vintage',size=4),
    }
    def name_get(self, cr, uid, ids, context=None):
        res = super(product_product,self).name_get(cr,uid,ids,context=context)
        result = []
        for nn in res:
            product = self.browse(cr, uid, nn[0], context=context)
            name = nn[1]
            if product.anyada:
                name = name + ' ' + product.anyada
            result.append((nn[0], name))
        return result



class product_template(osv.Model):
    _inherit = 'product.template'

    def _cellar_count(self, cr, uid, ids, field_name, arg, context=None):
        res = dict.fromkeys(ids, 0)
        for template in self.browse(cr, uid, ids, context=context):
            res[template.id] = sum([p.cellar_count for p in template.product_variant_ids])
        return res
    
    def action_view_sales(self, cr, uid, ids, context=None):
        act_obj = self.pool.get('ir.actions.act_window')
        mod_obj = self.pool.get('ir.model.data')
        product_ids = []
        for template in self.browse(cr, uid, ids, context=context):
            product_ids += [x.id for x in template.product_variant_ids]
        result = mod_obj.xmlid_to_res_id(cr, uid, 'sale.action_order_line_product_tree',raise_if_not_found=True)
        result = act_obj.read(cr, uid, [result], context=context)[0]
        result['domain'] = "[('product_id','in',[" + ','.join(map(str, product_ids)) + "]),('order_bodega_id','=',False)]"
        return result
    
    def action_view_cellar(self, cr, uid, ids, context=None):
        act_obj = self.pool.get('ir.actions.act_window')
        mod_obj = self.pool.get('ir.model.data')
        product_ids = []
        for template in self.browse(cr, uid, ids, context=context):
            product_ids += [x.id for x in template.product_variant_ids]
        result = mod_obj.xmlid_to_res_id(cr, uid, 'terranostravinos.action_cellar_line_product_tree',raise_if_not_found=True)
        result = act_obj.read(cr, uid, [result], context=context)[0]
        result['domain'] = "[('product_id','in',[" + ','.join(map(str, product_ids)) + "]),('order_bodega_id','!=',False)]"
        return result
    
    _columns = {
        'cellar_count': fields.function(_cellar_count, string='# Cellar', type='integer'),
    }


class sale_order_line_make_invoice(osv.osv_memory):
    _inherit = "sale.order.line.make.invoice"
    
    def make_invoices(self, cr, uid, ids, context=None):
        if context is None: context = {}
        res = False
        invoices = {}
        sales_order_line_obj = self.pool.get('sale.order.line')
        sales_order_obj = self.pool.get('sale.order')
        for line in sales_order_line_obj.browse(cr, uid, context.get('active_ids', []), context=context):
            if line.order_bodega_id:
                if line.order_id.invoice_ids:
                    ir_model_data = self.pool.get('ir.model.data')
                    form_res = ir_model_data.get_object_reference(cr, uid, 'account', 'invoice_form')
                    form_id = form_res and form_res[1] or False
                    tree_res = ir_model_data.get_object_reference(cr, uid, 'account', 'invoice_tree')
                    tree_id = tree_res and tree_res[1] or False
                    return {
                        'name': _('Invoice'),
                        'view_type': 'form',
                        'view_mode': 'form,tree',
                        'res_model': 'account.invoice',
                        'res_id': line.order_id.invoice_ids,
                        'view_id': False,
                        'views': [(form_id, 'form'), (tree_id, 'tree')],
                        'context': {'type': 'out_invoice'},
                        'type': 'ir.actions.act_window',
                    }
                else:
                    return True
        return super(sale_order_line_make_invoice, self).make_invoices(cr, uid, ids,context=context)
        

class commission_client_product_agent(osv.osv):
    _name = 'terranostravinos.commission.client.product.agent'
    _description = 'Commission client/product/agent'
    _columns = {
        'partner_id': fields.many2one('res.partner', 'Client', domain=[('customer', '=', True)]),
        'product_id': fields.many2one('product.product', 'Product'),
        'agent_id': fields.many2one('sale.agent', 'Agent'),
        'commission': fields.many2one('commission', 'Applied commission', required=True,
                                         help=''),        
        }
commission_client_product_agent()

class commission_client_product(osv.osv):
    _name = 'terranostravinos.commission.client.product'
    _description = 'Commission client/product'
    _columns = {
        'partner_id': fields.many2one('res.partner', 'Client', domain=[('customer', '=', True)]),
        'product_id': fields.many2one('product.product', 'Product'),
        'commission': fields.float('Commission'),
        }
commission_client_product()


class miestado(osv.osv):
    _name = 'terranostravinos.miestado'
    _description = 'Order Control State'
    _columns = {
        'name': fields.char('Name'),
        }

class facturas_bodega(osv.osv):
    _name = 'terranostravinos.cellar.order.invoices'
    _description = 'Cellar order invoices'
    _rec_name = 'factura_bodega_num'
    _columns = {
        'order_id': fields.many2one('sale.order', 'Order', readonly=True),
        'factura_bodega_num': fields.char('Cellar Invoice'),
        'factura_bodega_date': fields.date('Date Cellar Invoice'),
        'factura_bodega_venc': fields.date('Date Due Cellar Invoice'),
        'importe': fields.float('Amount'),
        'pagada': fields.boolean('Paid'),
        'partner_id': fields.related('order_id', 'partner_id', type='many2one', relation='res.partner', string='Customer', readonly=True),
        'bodega_id': fields.related('order_id', 'bodega_id', type='many2one', relation='res.partner', string='Cellar', readonly=True),
        }
    def onchange_bodega_factura_date(self, cr, uid, ids, date_invoice, payment_term_id, context=None):
        if not date_invoice:
            return {'value':{}}
        if date_invoice and not payment_term_id:
            # To make sure the invoice due date should contain due date which is
            # entered by user when there is no payment term defined
            return {'value': {'factura_bodega_venc': date_invoice}}
        pterm = self.pool.get('account.payment.term')
        pterm_list = pterm.compute(cr,uid,payment_term_id,value=1, date_ref=date_invoice)[0]
        if pterm_list:
            if isinstance(pterm_list,tuple):
                return {'value': {'factura_bodega_venc': pterm_list[0]}}       
            elif isinstance(pterm_list,list): 
                return {'value': {'factura_bodega_venc': pterm_list[0][0]}}        
        return {'value':{}}
